@@ -43,7 +43,6 @@
 // constructor
 nsLDAPOperation::nsLDAPOperation()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 // destructor
@@ -51,7 +50,7 @@ nsLDAPOperation::~nsLDAPOperation()
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsLDAPOperation, nsILDAPOperation);
+NS_IMPL_THREADSAFE_ISUPPORTS1(nsLDAPOperation, nsILDAPOperation)
 
 /**
  * Initializes this operation.  Must be called prior to use. 
@@ -61,7 +60,8 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsLDAPOperation, nsILDAPOperation);
  */
 NS_IMETHODIMP
 nsLDAPOperation::Init(nsILDAPConnection *aConnection,
-                      nsILDAPMessageListener *aMessageListener)
+                      nsILDAPMessageListener *aMessageListener,
+                      nsISupports *aClosure)
 {
     if (!aConnection) {
         return NS_ERROR_ILLEGAL_VALUE;
@@ -76,6 +76,7 @@ nsLDAPOperation::Init(nsILDAPConnection *aConnection,
     //
     mConnection = aConnection;
     mMessageListener = aMessageListener;
+    mClosure = aClosure;
 
     // cache the connection handle
     //
@@ -85,6 +86,23 @@ nsLDAPOperation::Init(nsILDAPConnection *aConnection,
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsLDAPOperation::GetClosure(nsISupports **_retval)
+{
+    if (!_retval) {
+        return NS_ERROR_ILLEGAL_VALUE;
+    }
+    NS_IF_ADDREF(*_retval = mClosure);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsLDAPOperation::SetClosure(nsISupports *aClosure)
+{
+    mClosure = aClosure;
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 nsLDAPOperation::GetConnection(nsILDAPConnection* *aConnection)
 {
@@ -98,6 +116,14 @@ nsLDAPOperation::GetConnection(nsILDAPConnection* *aConnection)
     return NS_OK;
 }
 
+void
+nsLDAPOperation::Clear()
+{
+  mMessageListener = nsnull;
+  mClosure = nsnull;
+  mConnection = nsnull;
+}
+
 NS_IMETHODIMP
 nsLDAPOperation::GetMessageListener(nsILDAPMessageListener **aMessageListener)
 {
@@ -114,24 +140,40 @@ nsLDAPOperation::GetMessageListener(nsILDAPMessageListener **aMessageListener)
 // wrapper for ldap_simple_bind()
 //
 NS_IMETHODIMP
-nsLDAPOperation::SimpleBind(const PRUnichar *passwd)
+nsLDAPOperation::SimpleBind(const nsACString& passwd)
 {
     nsresult rv;
-    nsXPIDLString bindName;
+    nsCAutoString bindName;
+    PRBool originalMsgID = mMsgID;
+    // Ugly hack alert:
+    // the first time we get called with a passwd, remember it.
+    // Then, if we get called again w/o a password, use the 
+    // saved one. Getting called again means we're trying to
+    // fall back to VERSION2.
+    // Since LDAP operations are thrown away when done, it won't stay
+    // around in memory.
+    if (!passwd.IsEmpty())
+      mSavePassword = passwd;
 
     NS_PRECONDITION(mMessageListener != 0, "MessageListener not set");
 
-    rv = mConnection->GetBindName(getter_Copies(bindName));
+    rv = mConnection->GetBindName(bindName);
     if (NS_FAILED(rv))
         return rv;
 
     PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, 
            ("nsLDAPOperation::SimpleBind(): called; bindName = '%s'; ",
-            NS_LossyConvertUCS2toASCII(bindName).get()));
+            bindName.get()));
 
-    mMsgID = ldap_simple_bind(mConnectionHandle,
-                              NS_ConvertUCS2toUTF8(bindName).get(),
-                              NS_ConvertUCS2toUTF8(passwd).get());
+    // Uf this is a second try at binding, remove the operation from pending ops
+    // because msg id has changed...
+    if (originalMsgID)
+      NS_STATIC_CAST(nsLDAPConnection *, 
+                        NS_STATIC_CAST(nsILDAPConnection *, 
+                        mConnection.get()))->RemovePendingOperation(this);
+
+    mMsgID = ldap_simple_bind(mConnectionHandle, bindName.get(),
+                              PromiseFlatCString(mSavePassword).get());
 
     if (mMsgID == -1) {
         const int lderrno = ldap_get_lderrno(mConnectionHandle, 0, 0);
@@ -159,7 +201,6 @@ nsLDAPOperation::SimpleBind(const PRUnichar *passwd)
   
     // make sure the connection knows where to call back once the messages
     // for this operation start coming in
-    //
     rv = NS_STATIC_CAST(nsLDAPConnection *, 
                         NS_STATIC_CAST(nsILDAPConnection *, 
                         mConnection.get()))->AddPendingOperation(this);
@@ -188,9 +229,9 @@ nsLDAPOperation::SimpleBind(const PRUnichar *passwd)
 // wrappers for ldap_search_ext
 //
 int
-nsLDAPOperation::SearchExt(const PRUnichar *base, // base DN to search
+nsLDAPOperation::SearchExt(const nsACString& base, // base DN to search
                            int scope, // SCOPE_{BASE,ONELEVEL,SUBTREE}
-                           const PRUnichar *filter, // search filter
+                           const nsACString& filter, // search filter
                            char **attrs, // attribute types to be returned
                            int attrsOnly, // attrs only, or values too?
                            LDAPControl **serverctrls, 
@@ -203,8 +244,8 @@ nsLDAPOperation::SearchExt(const PRUnichar *base, // base DN to search
         return NS_ERROR_NOT_INITIALIZED;
     }
 
-    return ldap_search_ext(mConnectionHandle, NS_ConvertUCS2toUTF8(base).get(),
-                           scope, NS_ConvertUCS2toUTF8(filter).get(), attrs,
+    return ldap_search_ext(mConnectionHandle, PromiseFlatCString(base).get(),
+                           scope, PromiseFlatCString(filter).get(), attrs,
                            attrsOnly, serverctrls, clientctrls, timeoutp,
                            sizelimit, &mMsgID);
 }
@@ -223,22 +264,23 @@ nsLDAPOperation::SearchExt(const PRUnichar *base, // base DN to search
  *
  * XXX doesn't currently handle LDAPControl params
  *
- * void searchExt(in string aBaseDn, in PRInt32 aScope,
- *                in string aFilter, PRUint32 aAttrCount,
+ * void searchExt(in AUTF8String aBaseDn, in PRInt32 aScope,
+ *                in AUTF8String aFilter, PRUint32 aAttrCount,
  *                const char **aAttributes, in PRIntervalTime aTimeOut,
  *                in PRInt32 aSizeLimit);
  */
 NS_IMETHODIMP
-nsLDAPOperation::SearchExt(const PRUnichar *aBaseDn, PRInt32 aScope, 
-                           const PRUnichar *aFilter, PRUint32 aAttrCount,
+nsLDAPOperation::SearchExt(const nsACString& aBaseDn, PRInt32 aScope, 
+                           const nsACString& aFilter, PRUint32 aAttrCount,
                            const char **aAttributes, PRIntervalTime aTimeOut,
                            PRInt32 aSizeLimit) 
 {
     PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, 
            ("nsLDAPOperation::SearchExt(): called with aBaseDn = '%s'; "
             "aFilter = '%s', aAttrCounts = %u, aSizeLimit = %d", 
-            NS_ConvertUCS2toUTF8(aBaseDn).get(), 
-            NS_ConvertUCS2toUTF8(aFilter).get(), aAttrCount, aSizeLimit));
+            PromiseFlatCString(aBaseDn).get(),
+            PromiseFlatCString(aFilter).get(),
+            aAttrCount, aSizeLimit));
 
     char **attrs = 0;
 
@@ -374,17 +416,24 @@ nsLDAPOperation::AbandonExt(LDAPControl **serverctrls,
     // succeeded (and there's nothing else the caller can reasonably do), 
     // so we only pay attention to this in debug builds.
     //
-    rv = NS_STATIC_CAST(nsLDAPConnection *, NS_STATIC_CAST(
-        nsILDAPConnection *, mConnection.get()))->RemovePendingOperation(this);
+    // check mConnection in case we're getting bit by 
+    // http://bugzilla.mozilla.org/show_bug.cgi?id=239729, wherein we 
+    // theorize that ::Clearing the operation is nulling out the mConnection
+    // from another thread.
+    if (mConnection)
+    {
+      rv = NS_STATIC_CAST(nsLDAPConnection *, NS_STATIC_CAST(
+          nsILDAPConnection *, mConnection.get()))->RemovePendingOperation(this);
 
-    if (NS_FAILED(rv)) {
-        // XXXdmose should we use keep Abandon from happening on multiple 
-        // threads at the same time?  that's when this condition is most 
-        // likely to occur.  i _think_ the LDAP C SDK is ok with this; need 
-        // to verify.
-        //
-        NS_WARNING("nsLDAPOperation::AbandonExt: "
-                   "mConnection->RemovePendingOperation(this) failed.");
+      if (NS_FAILED(rv)) {
+          // XXXdmose should we use keep Abandon from happening on multiple 
+          // threads at the same time?  that's when this condition is most 
+          // likely to occur.  i _think_ the LDAP C SDK is ok with this; need 
+          // to verify.
+          //
+          NS_WARNING("nsLDAPOperation::AbandonExt: "
+                     "mConnection->RemovePendingOperation(this) failed.");
+      }
     }
 
     return retStatus;
