@@ -44,10 +44,11 @@
 #include "prthread.h"
 #include "plstr.h"
 #include "prprf.h"
-#include "nsIFileStream.h"
-#include "nsFileSpec.h"
+#include "nsIFile.h"
 #include "nsCOMPtr.h"
+#ifdef OJI
 #include "nsIJVMManager.h"
+#endif
 #include "nsIServiceManager.h"
 
 #include "nsIDocument.h"
@@ -58,9 +59,9 @@
 #include "nsIFileStreams.h"
 #include "nsNetUtil.h"
 
-#if defined(XP_PC) && !defined(XP_OS2)
-#include "windows.h"
-#include "winbase.h"
+#ifdef XP_WIN
+#include <windows.h>
+#include <winbase.h>
 #endif
 
 nsPluginInstancePeerImpl::nsPluginInstancePeerImpl()
@@ -83,7 +84,8 @@ nsPluginInstancePeerImpl::~nsPluginInstancePeerImpl()
 
 static NS_DEFINE_IID(kIPluginTagInfoIID, NS_IPLUGINTAGINFO_IID); 
 static NS_DEFINE_IID(kIPluginTagInfo2IID, NS_IPLUGINTAGINFO2_IID); 
-static NS_DEFINE_IID(kIJVMPluginTagInfoIID, NS_IJVMPLUGINTAGINFO_IID); 
+#ifdef OJI
+static NS_DEFINE_IID(kIJVMPluginTagInfoIID, NS_IJVMPLUGINTAGINFO_IID);
 
 NS_IMPL_ISUPPORTS7(nsPluginInstancePeerImpl,
                    nsIPluginInstancePeer,
@@ -92,7 +94,16 @@ NS_IMPL_ISUPPORTS7(nsPluginInstancePeerImpl,
                    nsIPluginTagInfo,
                    nsIPluginTagInfo2,
                    nsIJVMPluginTagInfo,
-                   nsPIPluginInstancePeer);
+                   nsPIPluginInstancePeer)
+#else
+NS_IMPL_ISUPPORTS6(nsPluginInstancePeerImpl,
+                   nsIPluginInstancePeer,
+                   nsIPluginInstancePeer2,
+                   nsIWindowlessPluginInstancePeer,
+                   nsIPluginTagInfo,
+                   nsIPluginTagInfo2,
+                   nsPIPluginInstancePeer)
+#endif
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::GetValue(nsPluginInstancePeerVariable variable, void *value)
 {
@@ -146,8 +157,8 @@ protected:
   nsIPluginInstanceOwner* mOwner;
 };
 
-NS_IMPL_ADDREF(nsPluginStreamToFile);
-NS_IMPL_RELEASE(nsPluginStreamToFile);
+NS_IMPL_ADDREF(nsPluginStreamToFile)
+NS_IMPL_RELEASE(nsPluginStreamToFile)
 
 nsPluginStreamToFile::nsPluginStreamToFile(const char* target, nsIPluginInstanceOwner* owner) :
   mTarget(PL_strdup(target)),
@@ -597,6 +608,7 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetUniqueID(PRUint32 *result)
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::GetCode(const char* *result)
 {
+#ifdef OJI
   if (nsnull != mOwner) {
     nsIJVMPluginTagInfo *tinfo;
     nsresult            rv;
@@ -611,13 +623,17 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetCode(const char* *result)
     return rv;
   }
   else {
+#endif
     *result = 0;
     return NS_ERROR_FAILURE;
+#ifdef OJI
   }
+#endif
 }
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::GetCodeBase(const char* *result)
 {
+#ifdef OJI
   if (nsnull != mOwner) {
     nsIJVMPluginTagInfo *tinfo;
     nsresult            rv;
@@ -632,13 +648,17 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetCodeBase(const char* *result)
     return rv;
   }
   else {
+#endif
     *result = 0;
     return NS_ERROR_FAILURE;
+#ifdef OJI
   }
+#endif
 }
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::GetArchive(const char* *result)
 {
+#ifdef OJI
   if (nsnull != mOwner) {
     nsIJVMPluginTagInfo *tinfo;
     nsresult            rv;
@@ -653,13 +673,17 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetArchive(const char* *result)
     return rv;
   }
   else {
+#endif
     *result = 0;
     return NS_ERROR_FAILURE;
+#ifdef OJI
   }
+#endif
 }
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::GetName(const char* *result)
 {
+#ifdef OJI
   if (nsnull != mOwner) {
     nsIJVMPluginTagInfo *tinfo;
     nsresult            rv;
@@ -674,13 +698,17 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetName(const char* *result)
     return rv;
   }
   else {
+#endif
     *result = 0;
     return NS_ERROR_FAILURE;
+#ifdef OJI
   }
+#endif
 }
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::GetMayScript(PRBool *result)
 {
+#ifdef OJI
   if (nsnull != mOwner) {
     nsIJVMPluginTagInfo *tinfo;
     nsresult            rv;
@@ -695,9 +723,12 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetMayScript(PRBool *result)
     return rv;
   }
   else {
+#endif
     *result = 0;
     return NS_ERROR_FAILURE;
+#ifdef OJI
   }
+#endif
 }
 
 NS_IMETHODIMP nsPluginInstancePeerImpl::SetWindowSize(PRUint32 width, PRUint32 height)
@@ -714,8 +745,7 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetJSWindow(JSObject* *outJSWindow)
   rv = mOwner->GetDocument(getter_AddRefs(document));
 
   if (NS_SUCCEEDED(rv) && document) {
-    nsCOMPtr<nsIScriptGlobalObject> global;
-    document->GetScriptGlobalObject(getter_AddRefs(global));
+    nsIScriptGlobalObject *global = document->GetScriptGlobalObject();
 
     if(global) {
       *outJSWindow = global->GetGlobalJSObject();
@@ -740,17 +770,13 @@ NS_IMETHODIMP nsPluginInstancePeerImpl::GetJSContext(JSContext* *outContext)
   rv = mOwner->GetDocument(getter_AddRefs(document));
 
   if (NS_SUCCEEDED(rv) && document) {
-    nsCOMPtr<nsIScriptGlobalObject> global;
-    document->GetScriptGlobalObject(getter_AddRefs(global));
+    nsIScriptGlobalObject *global = document->GetScriptGlobalObject();
 
     if (global) {
-      nsCOMPtr<nsIScriptContext> context;
+      nsIScriptContext *context = global->GetContext();
 
-      if (global->GetContext(getter_AddRefs(context)) == NS_OK) {
-        if (context) {
-            *outContext = (JSContext*) context->GetNativeContext();
-            rv = NS_OK;
-        }
+      if (context) {
+        *outContext = (JSContext*) context->GetNativeContext();
       }
     }
   }
