@@ -45,8 +45,8 @@ static SECStatus ChangeTrustAttributes(CERTCertDBHandle *db,
 	CERTCertificate *cert, char *trusts);
 static SECStatus set_cert_type(CERTCertificate *cert, unsigned int type);
 static SECItem *sign_cert(CERTCertificate *cert, SECKEYPrivateKey *privk);
-static CERTCertificate* install_cert(CERTCertDBHandle *db, PK11SlotInfo *slot,
-            SECItem *derCert, char *nickname);
+static CERTCertificate* install_cert(CERTCertDBHandle *db, SECItem *derCert,
+            char *nickname);
 static SECStatus GenerateKeyPair(PK11SlotInfo *slot, SECKEYPublicKey **pubk,
 	SECKEYPrivateKey **privk, int keysize);
 static CERTCertificateRequest* make_cert_request(char *subject,
@@ -108,6 +108,7 @@ GenerateCert(char *nickname, int keysize, char *token)
 
 	if(cert) {
 		output_ca_cert(cert, db);
+		CERT_DestroyCertificate(cert);
 	}
 
 	PORT_Free(subject);
@@ -333,13 +334,15 @@ GenerateSelfSignedObjectSigningCert(char *nickname, CERTCertDBHandle *db,
 	}
 
 	derCert = sign_cert (temp_cert, privk);
-	cert = install_cert(db, slot, derCert, nickname);
+	cert = install_cert(db, derCert, nickname);
 	if(ChangeTrustAttributes(db, cert, ",,uC") != SECSuccess) {
 		FatalError("Unable to change trust on generated certificate");
 	}
 
 	/* !!! Free memory ? !!! */
 	PK11_FreeSlot(slot);
+	SECKEY_DestroyPrivateKey(privk);
+	SECKEY_DestroyPublicKey(pubk);
 
 	return cert;
 }
@@ -425,7 +428,7 @@ sign_cert(CERTCertificate *cert, SECKEYPrivateKey *privk)
   SECItem *result2;
 
   void *dummy;
-  SECOidTag alg;
+  SECOidTag alg = SEC_OID_UNKNOWN;
 
   switch (privk->keyType) 
     {
@@ -439,6 +442,7 @@ sign_cert(CERTCertificate *cert, SECKEYPrivateKey *privk)
 	default:
 		FatalError("Unknown key type");
     }
+  PORT_Assert(alg != SEC_OID_UNKNOWN);
 
   rv = SECOID_SetAlgorithmID (cert->arena, &cert->signature, alg, 0);
 
@@ -491,15 +495,12 @@ sign_cert(CERTCertificate *cert, SECKEYPrivateKey *privk)
  * Installs the cert in the permanent database.
  */
 static CERTCertificate*
-install_cert(CERTCertDBHandle *db, PK11SlotInfo *slot, SECItem *derCert,
-        char *nickname)
+install_cert(CERTCertDBHandle *db, SECItem *derCert, char *nickname)
 {
 	CERTCertificate *newcert;
-	CERTCertTrust trust;
     PK11SlotInfo *newSlot;
 
-	newcert = CERT_NewTempCertificate(db, derCert, NULL,
-	 /*isperm*/ PR_FALSE, /*copyDER*/ PR_TRUE);
+	newcert = CERT_DecodeDERCertificate(derCert, PR_TRUE, NULL);
 
 	if (newcert == NULL) {
 		PR_fprintf(errorFD, "%s: can't create new certificate\n", PROGRAM_NAME);
@@ -508,31 +509,12 @@ install_cert(CERTCertDBHandle *db, PK11SlotInfo *slot, SECItem *derCert,
 	}
 
     newSlot = PK11_ImportCertForKey(newcert, nickname, NULL /*wincx*/);
-    if( slot == NULL ) {
+    if( newSlot == NULL ) {
         PR_fprintf(errorFD, "Unable to install certificate\n");
         errorCount++;
         exit(ERRX);
     }
-
-	PORT_Memset ((void *) &trust, 0, sizeof(trust));
-	trust.objectSigningFlags |= CERTDB_USER;
-
-    if( newSlot == PK11_GetInternalKeySlot() ) {
-        /* newcert is now a permanent cert */
-        if( CERT_ChangeCertTrust(db, newcert, &trust) != SECSuccess) {
-            PR_fprintf(errorFD,
-                "Failed to change trust of generated certificate\n");
-            errorCount++;
-            exit(ERRX);
-        }
-    } else {
-	    if (CERT_AddTempCertToPerm (newcert, nickname, &trust) != SECSuccess) {
-		    PR_fprintf(errorFD, "%s: Failure adding \"%s\" certificate to "
-		        "permanent DB\n", PROGRAM_NAME, nickname);
-		    errorCount++;
-		    exit (ERRX);
-	    } 
-    }
+    PK11_FreeSlot(newSlot);
 
     if(verbosity >= 0){
 	   PR_fprintf(outputFD, "certificate \"%s\" added to database\n", nickname);
@@ -691,11 +673,13 @@ output_ca_cert (CERTCertificate *cert, CERTCertDBHandle *db)
   certChain = SEC_PKCS7CreateCertsOnly (cert, PR_TRUE, db);
   encodedCertChain 
      = SEC_PKCS7EncodeItem (NULL, NULL, certChain, NULL, NULL, NULL);
+  SEC_PKCS7DestroyContentInfo (certChain);
 
   if (encodedCertChain) 
     {
     fprintf(out, "Content-type: application/x-x509-ca-cert\n\n");
     fwrite (encodedCertChain->data, 1, encodedCertChain->len, out);
+    SECITEM_FreeItem(encodedCertChain, PR_TRUE);
     }
   else {
     PR_fprintf(errorFD, "%s: Can't DER encode this certificate\n", PROGRAM_NAME);
