@@ -46,11 +46,13 @@
 
 #include "nsISample.h"
 #include "nsIServiceManager.h"
-#include "nsXPIDLString.h"
 #include "nsIComponentRegistrar.h"
 
 #ifdef XPCOM_GLUE
 #include "nsXPCOMGlue.h"
+#include "nsMemory.h"
+#else
+#include "nsXPIDLString.h"
 #endif
 
 #define NS_SAMPLE_CONTRACTID "@mozilla.org/sample;1"
@@ -114,8 +116,14 @@ main(void)
         return -3;
     }
     printf("Set value to: %s\n", testValue);
+#ifndef XPCOM_GLUE
     nsXPIDLCString str;
     rv = mysample->GetValue(getter_Copies(str));
+#else
+    char *str;
+    rv = mysample->GetValue(&str);
+#endif
+
     if (NS_FAILED(rv))
     {
         printf("ERROR: Calling nsISample::GetValue() [%x]\n", rv);
@@ -127,6 +135,9 @@ main(void)
         return -4;
     }
 
+#ifdef XPCOM_GLUE
+    nsMemory::Free(str);
+#endif
     rv = mysample->WriteValue("Final print :");
     printf("Test passed.\n");
     
